@@ -1,3 +1,4 @@
+# 2014-09-01 CJS Change Inf to NA 
 # 2012-01-22 CJS Made X/Y axis limits the same so that Bayesian p-value prints properly
 # 2011-06-13 CJS returned p-values
 # 2010-03-29 CJS First creation of routine
@@ -14,6 +15,11 @@ PredictivePosteriorPlot.TSPDE.WHCH2 <- function( discrep  ) {
 #     (13-14)  o,s Freeman-Tukey for u2.A.1  +u2.N.1
 #     (15-16)  o,s Freeman-Tukey for all data (m2, YoY and Age 1)`
 
+# Change any Inf to NA
+temp <- discrep == Inf | discrep == -Inf
+if(sum(temp)>0){cat(sum(temp), " infinite discrepancy measures set to NA\n")}
+discrep[ temp ] <- NA
+
 #browser()
 titles <- c("Freeman-Tukey for m2", 
             "Freeman-Tukey for u2.A.YoY", 
@@ -42,7 +48,7 @@ for(page in 1:2){
      abline(a=0, b=1)
 
      ## Compute Bayesian p-value
-     p.value <- sum(discrep[,(page-1)*4+2*i-1]<discrep[,(page-1)*4+2*i])/nrow(discrep)
+     p.value <- sum(discrep[,(page-1)*4+2*i-1]<discrep[,(page-1)*4+2*i], na.rm=TRUE)/nrow(discrep)
      saved_p_values[i] <- p.value
      
   ## Add p-value to plot
