@@ -1,3 +1,4 @@
+# 2014-09-01 CJS dealing with Inf and -Inf in the discrepancy measures
 # 2012-01-22 CJS made X/Y axis limits the same so p-value prints properly
 # 2011-06-13 CJS returned bayesian p-values
 
@@ -9,7 +10,12 @@ PredictivePosteriorPlot.TSPNDE <- function( discrep  ) {
 #     (5-6)  o,s Freeman-Tukey measures for u2
 #     (7-8)  o,s Deviance for u2
 #      9-10  o,s Freeman-Tukey for m2+u2
-#     11-12  o,s Deviance for m2+uw
+#     11-12  o,s Deviance for m2+u2
+
+# Change any Inf to NA
+temp <- discrep == Inf | discrep == -Inf
+if(sum(temp)>0){cat(sum(temp), " infinite discrepancy measures set to NA\n")}
+discrep[ temp ] <- NA
 
 split.screen(figs=c(3,2))  # 3 rows and 2 columns
 titles <- c("Freeman-Tukey for m2", 
@@ -26,7 +32,7 @@ for(i in 1:6){
   par(mai=c(.40,.40,.40,.40)) # margins of plot relative to plotting position
 
   ## Compute plotting limits to generate symmetric plot
-  lims <- range(discrep[,(2*i):(2*i-1)])
+  lims <- range(discrep[,(2*i):(2*i-1)], na.rm=TRUE)
   
   ## Plot observed vs simulated discrepancies
   plot(discrep[,(2*i):(2*i-1)],
@@ -35,12 +41,12 @@ for(i in 1:6){
   abline(a=0, b=1)
 
   ## Compute Bayesian p-value
-  p.value <- sum(discrep[,2*i-1]<discrep[,2*i])/nrow(discrep)
+  p.value <- sum(discrep[,2*i-1]<discrep[,2*i], na.rm=TRUE)/nrow(discrep)
   saved_p_values[i] <- p.value
 
   ## Add p-value to plot
-  x.loc <- mean(lims)
-  y.loc <- min(lims)
+  x.loc <- mean(lims, na.rm=TRUE)
+  y.loc <- min(lims,  na.rm=TRUE)
   
   text(x.loc, y.loc,
        labels=paste("Bayesian GOF P:",formatC(p.value, digits=2, format="f")),
