@@ -1,5 +1,6 @@
 #
-# 2009-12-05
+# 2009-12-05 CJS first editions
+# 2014-09-01 CJS remove prompts; jags; engine
 # This is a demonstration of how to call the Time Stratified Petersen with Diagonal Entries (TSPDE) program
 # SEPARATING WILD VS HATCHERY CHINOOK SALMON.
 
@@ -47,30 +48,13 @@
 #  The prefix is used to identify the output files for this run.
 #  The title  is used to title the output.
 
-## Warn user that demo may overwrite existing files
-demo.proceed <- TRUE
-if(!exists("demo.ans")) demo.ans <- " "
-
-while(! demo.ans %in% c("yes","no","YES","NO","Y","N","y","n")){
-  cat("***** WARNING ***** This demonstration may create/over-write objects with names 'demo.xxx' \n")
-  cat("***** WARNING ***** This demonstration may create/over-write a directory 'demo-TSPDE-WHchinook' \n")
-  demo.ans <- readline(prompt="Do you want to proceed? (yes/no): ")
-}
-if(demo.ans %in% c("no","NO","n","N")){demo.proceed <- FALSE }
-
 # Create a directory to store the results Test and then create the
 # directory
 if(file.access("demo-TSPDE-WHchinook")!=0){
-  demo.proceed <- demo.proceed & dir.create("demo-TSPDE-WHchinook", showWarnings=TRUE)
+  dir.create("demo-TSPDE-WHchinook", showWarnings=TRUE)
 }
 setwd("demo-TSPDE-WHchinook")
 
-if(!demo.proceed){stop()}
-
-
-par(ask=FALSE)
-dev.off()  # turn off the blank graphics window
-
 ## Load BTSPAS library
 library(BTSPAS)
 
@@ -155,6 +139,7 @@ demo.jc.2003.ch.tspde <- TimeStratPetersenDiagErrorWHChinook_fit(
                   clip.frac.H= demo.clip.frac.H,
                   hatch.after=demo.hatch.after,
                   bad.m2=demo.bad.m2, bad.u2.A=demo.bad.u2.A, bad.u2.N=demo.bad.u2.N,
+		  #engine="openbugs",   # show how to call openbugs
                   debug=TRUE  # this generates only 10,000 iterations of the MCMC chain for checking.
                   )
 
