@@ -1,4 +1,6 @@
-# 2009-12-06
+# 2009-12-06 CJS First edition
+# 2014-09-01 CJS remove prompts; jags; engine
+
 # This is a demonstration of how to call the Time Stratified Petersen with Diagonal Entries (TSPDE) program
 # SEPARATING WILD VS HATCHERY STEELHEAD SALMON.
 #
@@ -33,31 +35,13 @@
 #    Marking and releasing of steelhead took place in only a few weeks! The hierarchical model
 #    will extrapolate outside these weeks to estimate the capture rate.
 
-## Warn user that demo may overwrite existing files
-demo.proceed <- TRUE
-if(!exists("demo.ans")) demo.ans <- " "
-
-while(! demo.ans %in% c("yes","no","YES","NO","Y","N","y","n")){
-  cat("***** WARNING ***** This demonstration may create/over-write objects with names 'demo.xxx' \n")
-  cat("***** WARNING ***** This demonstration may create/over-write a directory 'demo-demo-TSPDE-WHsteel' \n")
-  demo.ans <- readline(prompt="Do you want to proceed? (yes/no): ")
-}
-if(demo.ans %in% c("no","NO","n","N")){demo.proceed <- FALSE }
-
 # Create a directory to store the results Test and then create the
 # directory
 if(file.access("demo-demo-TSPDE-WHsteel")!=0){
-  demo.proceed <- demo.proceed & dir.create("demo-demo-TSPDE-WHsteel", showWarnings=TRUE)
+  dir.create("demo-demo-TSPDE-WHsteel", showWarnings=TRUE)
 }
 setwd("demo-demo-TSPDE-WHsteel")
 
-if(!demo.proceed){stop()}
-
-
-par(ask=FALSE)
-dev.off()  # turn off the blank graphics window
-
-
 ## Load BTSPAS library
 library(BTSPAS)
 
@@ -145,6 +129,7 @@ demo.jc.2003.st.tspde <- TimeStratPetersenDiagErrorWHSteel_fit(
                   bad.u2.W.YoY=demo.bad.u2.W.YoY,
                   bad.u2.W.1  =demo.bad.u2.W.1,
                   bad.u2.H.1  =demo.bad.u2.H.1,
+		  #engine="openbugs", # show how to call openbugs
                   debug=TRUE  # this generates only 10,000 iterations of the MCMC chain for checking.
                   )
 
